@@ -46,9 +46,9 @@ function _formatTimeString(timeVal) {
     return str;
 }
 
-const AMAZON_IMAGE_FORMAT = "http://images.amazon.com/images/P/%s.01.LZZZZZZZ.jpg";
-const MUSIC_BRAINZ_ASIN_FORMAT = "https://musicbrainz.org/ws/2/release/?query=release:\"%s\"AND artist:\"%s\"&limit=1&fmt=json&inc=asin";
-const fetchCoverArt = function(_tagList, _callback) {
+const COVER_ART_ARCHIVE_URL = "https://coverartarchive.org/release/%s";
+const MUSIC_BRAINZ_ASIN_FORMAT = "https://musicbrainz.org/ws/2/release/?query=release:\"%s\"AND artist:\"%s\"&limit=1&fmt=json";
+const fetchCoverArt = function(_tagList, _cancellable, _callback) {
     function _fetchFromTags() {
         let coverSample = null;
         let idx = 0;
@@ -85,13 +85,13 @@ const fetchCoverArt = function(_tagList, _callback) {
         return null;
     }
 
-    function _getCacheFile(asin) {
+    function _getCacheFile(mbid) {
         let cachePath = GLib.build_filenamev([GLib.get_user_cache_dir(), 'sushi']);
-        return Gio.File.new_for_path(GLib.build_filenamev([cachePath, `${asin}.jpg`]));
+        return Gio.File.new_for_path(GLib.build_filenamev([cachePath, `${mbid}.jpg`]));
     }
 
     function _fetchFromStream(stream, done) {
-        GdkPixbuf.Pixbuf.new_from_stream_async(stream, null, (o, res) => {
+        GdkPixbuf.Pixbuf.new_from_stream_async(stream, _cancellable, (o, res) => {
             let cover;
             try {
                 cover = GdkPixbuf.Pixbuf.new_from_stream_finish(res);
@@ -104,9 +104,9 @@ const fetchCoverArt = function(_tagList, _callback) {
         });
     }
 
-    function _fetchFromCache(asin, done) {
-        let file = _getCacheFile(asin);
-        file.query_info_async(Gio.FILE_ATTRIBUTE_STANDARD_TYPE, 0, 0, null, (f, res) => {
+    function _fetchFromCache(mbid, done) {
+        let file = _getCacheFile(mbid);
+        file.query_info_async(Gio.FILE_ATTRIBUTE_STANDARD_TYPE, 0, 0, _cancellable, (f, res) => {
             try {
                 file.query_info_finish(res);
             } catch (e) {
@@ -114,7 +114,7 @@ const fetchCoverArt = function(_tagList, _callback) {
                 return;
             }
 
-            file.read_async(0, null, (f, res) => {
+            file.read_async(0, _cancellable, (f, res) => {
                 let stream;
                 try {
                     stream = file.read_finish(res);
@@ -128,12 +128,12 @@ const fetchCoverArt = function(_tagList, _callback) {
         });
     }
 
-    function _saveToCache(asin, stream, done) {
-        let cacheFile = _getCacheFile(asin);
+    function _saveToCache(mbid, stream, done) {
+        let cacheFile = _getCacheFile(mbid);
         let cachePath = cacheFile.get_parent().get_path();
         GLib.mkdir_with_parents(cachePath, 448);
 
-        cacheFile.replace_async(null, false, Gio.FileCreateFlags.PRIVATE, 0, null, (f, res) => {
+        cacheFile.replace_async(null, false, Gio.FileCreateFlags.PRIVATE, 0, _cancellable, (f, res) => {
             let outStream;
             try {
                 outStream = cacheFile.replace_finish(res);
@@ -146,7 +146,7 @@ const fetchCoverArt = function(_tagList, _callback) {
                 stream,
                 Gio.OutputStreamSpliceFlags.CLOSE_SOURCE |
                 Gio.OutputStreamSpliceFlags.CLOSE_TARGET,
-                0, null, (s, res) => {
+                0, _cancellable, (s, res) => {
                     try {
                         outStream.splice_finish(res);
                     } catch (e) {
@@ -159,68 +159,90 @@ const fetchCoverArt = function(_tagList, _callback) {
         });
     }
 
-    function _fetchFromAmazon(asin, done) {
-        let uri = AMAZON_IMAGE_FORMAT.format(asin);
+    function decode(buffer) {
+        let decoder = new TextDecoder('utf8');
+        return decoder.decode(buffer);
+    }
+
+    function _fetchCoverArtArchiveImage(uri, mbid, done) {
         let session = new Soup.Session();
 
-        let request;
-        try {
-            request = session.request(uri);
-        } catch (e) {
-            done(e, null);
-            return;
-        }
+        let message = Soup.Message.new('GET', uri);
+        message.request_headers.append('User-Agent', 'gnome-sushi');
 
-        request.send_async(null, (r, res) => {
+        session.send_async(message, 0, _cancellable, (r, res) => {
             let stream;
             try {
-                stream = request.send_finish(res);
+                stream = session.send_finish(res);
             } catch (e) {
                 done(e, null);
                 return;
             }
 
-            _saveToCache(asin, stream, (err) => {
+            _saveToCache(mbid, stream, (err) => {
                 if (err)
                     logError(err, 'Unable to save cover to cache');
-                _fetchFromCache(asin, done);
+                _fetchFromCache(mbid, done);
             });
         });
     }
 
-    function _fetchFromASIN(done) {
+    function _fetchCoverArtArchiveMetadata(mbid, done) {
+        let uri = COVER_ART_ARCHIVE_URL.format(mbid);
+        let session = new Soup.Session();
+
+        let message = Soup.Message.new('GET', uri);
+        message.request_headers.append('User-Agent', 'gnome-sushi');
+        session.send_and_read_async(message, 0, _cancellable, (r, res) => {
+            try {
+                let data = decode(session.send_and_read_finish(res).get_data());
+                if (message.get_status() !== Soup.Status.OK)
+                  return;
+
+                let json_data = JSON.parse (data);
+
+                let uri = json_data['images'][0]['thumbnails']['small'];
+                _fetchCoverArtArchiveImage(uri, mbid, done);
+                return;
+            } catch (e) {
+                done(e, null);
+            }
+        });
+    }
+
+    function _fetchFromMusicBrainz(done) {
         let artist = _tagList.get_string('artist')[1];
         let album = _tagList.get_string('album')[1];
 
         let uri = MUSIC_BRAINZ_ASIN_FORMAT.format(album, artist);
-        let session = new Soup.SessionAsync();
+        let session = new Soup.Session();
 
-        let request;
-        try {
-            request = Soup.Message.new('GET', uri);
-            request.request_headers.append('User-Agent', 'gnome-sushi');
-        } catch (e) {
-            done(e, null);
-            return;
-        }
+        let message = Soup.Message.new('GET', uri);
+        message.request_headers.append('User-Agent', 'gnome-sushi');
 
-        session.queue_message(request, (r, res) => {
-            let asin = null;
-            if (request.status_code == Soup.Status.OK) {
-                try {
-                    let json_response = JSON.parse(request.response_body.data);
-                    asin = json_response['release'][0]['asin'].toString();
-                } catch (e) {
-                    done(e, null);
+        session.send_and_read_async(message, 0, _cancellable, (r, res) => {
+            let mbid = null;
+            try {
+                let data = decode(session.send_and_read_finish(res).get_data());
+                if (message.get_status() !== Soup.Status.OK)
                     return;
-                }
+
+                let json_response = JSON.parse(data);
+
+                if (!('releases' in json_response) || json_response['releases'].length === 0)
+                    return;
+
+                mbid = json_response['releases'][0]['id'];
+            } catch (e) {
+              done (e, null);
+              return;
             }
 
-            _fetchFromCache(asin, (err, cover) => {
+            _fetchFromCache(mbid, (err, cover) => {
                 if (cover)
                     done(null, cover);
                 else
-                    _fetchFromAmazon(asin, done);
+                    _fetchCoverArtArchiveMetadata(mbid, done);
             });
         });
     }
@@ -231,7 +253,7 @@ const fetchCoverArt = function(_tagList, _callback) {
        return;
    }
 
-    _fetchFromASIN(_callback);
+    _fetchFromMusicBrainz(_callback);
 }
 
 const AudioPlayer = GObject.registerClass({
@@ -321,6 +343,7 @@ var Klass = GObject.registerClass({
         vbox.pack_start(this._albumLabel, false, false, 0);
 
         this.connect('destroy', this._onDestroy.bind(this));
+        this._cancellable = new Gio.Cancellable();
         this.isReady();
     }
 
@@ -329,6 +352,8 @@ var Klass = GObject.registerClass({
             GLib.source_remove(this._autoplayId);
             this._autoplayId = 0;
         }
+
+        this._cancellable.cancel();
     }
 
     _setCover(cover) {
@@ -352,7 +377,8 @@ var Klass = GObject.registerClass({
 
     _onCoverArtFetched(err, cover) {
         if (err) {
-            if (!err.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND))
+            if (!err.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND) &&
+                !err.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED))
                 logError(err, 'Unable to fetch cover art');
             return;
         }
@@ -384,7 +410,7 @@ var Klass = GObject.registerClass({
         this._titleLabel.set_markup('<b>' + escaped + '</b>');
 
         if (artistName && albumName && !this._coverFetched) {
-            fetchCoverArt(tags, this._onCoverArtFetched.bind(this));
+            fetchCoverArt(tags, this._cancellable, this._onCoverArtFetched.bind(this));
             this._coverFetched = true;
         }
     }
