@@ -49,15 +49,23 @@ var Klass = GObject.registerClass({
     _init(file, fileInfo) {
         super._init();
 
+        this._cancellable = new Gio.Cancellable();
+
         let buffer = this._createBuffer(file, fileInfo);
         this._view = new GtkSource.View({ buffer: buffer,
                                           editable: false,
                                           cursor_visible: false,
                                           monospace: true,
                                           show_line_numbers: !!buffer.language });
-        this._view.set_can_focus(false);
+
         this.add(this._view);
         this.isReady();
+
+        this.connect('destroy', this._onDestroy.bind(this));
+    }
+
+    _onDestroy() {
+        this._cancellable.cancel();
     }
 
     _createBuffer(file, fileInfo) {
@@ -80,11 +88,12 @@ var Klass = GObject.registerClass({
         let sourceFile = new GtkSource.File({ location: file });
         let loader = new GtkSource.FileLoader({ buffer: buffer,
                                                 file: sourceFile });
-        loader.load_async(0, null, null, (loader, result) => {
+        loader.load_async(0, this._cancellable, null, (loader, result) => {
             try {
                 loader.load_finish(result);
             } catch (e) {
-                this.emit('error', e);
+                if(!e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED))
+                    this.emit('error', e);
             }
         });
 
